#                              Copyright (C) 2023, UChicago Argonne, LLC
# 
#                                         All Rights Reserved
# 
#         Software Name: SONICC (Scale-dependent Omega for Neutrino Initial Condition Codes)
# 
#                 By: Argonne National Laboratory, University of California, Berkeley
# 
#                                         OPEN SOURCE LICENSE
# 
# Redistribution and use in source and binary forms, with or without modification, are permitted
# provided that the following conditions are met:
# 
#   1. Redistributions of source code must retain the above copyright notice, this list of conditions
#      and the following disclaimer.
# 
#   2. Redistributions in binary form must reproduce the above copyright notice, this list of
#      conditions and the following disclaimer in the documentation and/or other materials provided
#      with the distribution.
# 
#   3. Neither the name of the copyright holder nor the names of its contributors may be used to
#      endorse or promote products derived from this software without specific prior written
#      permission.
#
# ***************************************************************************************************
#                                              DISCLAIMER
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
# IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# ***************************************************************************************************

from . import IO
import time


class Timer:
    """
    Class containing timing containers.
    """

    def __init__(self):

        # Instantiate dictionary that will contain start and stop times
        self.times = {}
        self.ntimes = 0

    def StartTime(self, message=None):
        """
        Start timer for new key in the dictionary. Optionally print message to screen
        """

        key = self.ntimes
        self.times[key] = time.time()
        self.ntimes += 1

        if message is not None:
            print(message)

        return key

    def PrintElapsedTime(self, key):
        """
        Print the elapsed time for this key.
        """

        IO.CheckError(key in self.times, "Timer key does not exist: {:d}".format(key))

        telapsed = time.time() - self.times[key]
        del self.times[key]

        print("Done!\nElapsed time: {:8.2f}\n".format(telapsed))

    def ElapsedTime(self, key):
        """
        Return the elapsed time for this key.
        """

        IO.CheckError(key in self.times, "Timer key does not exist: {:d}".format(key))

        telapsed = time.time() - self.times[key]
        del self.times[key]

        return telapsed
